#ifndef Tracker_APVAnalysis_h
#define Tracker_APVAnalysis_h


class TkApvMask;
class TkZeroSuppresser;
class TkCommonModeCalculator;
class TkPedestalCalculator;
class ApvEventReader;
class TkNoiseCalculator;

#include <vector>
#include <utility>

class ApvAnalysis 
{
 public:
  typedef vector<unsigned short> RawSignalType;
  typedef vector<float> SubtractedSignalType;
  typedef vector<pair <short,float> > ZeroSuppressedSignalType;

  
  
  void setZeroSuppresser (TkZeroSuppresser& in ){theTkZeroSuppresser = &in;} 
  void setCommonModeCalculator (TkCommonModeCalculator& in ){theTkCommonModeCalculator = &in;} 
  void setPedestalCalculator (TkPedestalCalculator& in ){theTkPedestalCalculator = &in;} 
  void setNoiseCalculator(TkNoiseCalculator& in){theTkNoiseCalculator = &in;} 
  void setMask(TkApvMask& in){theTkApvMask=&in;}
  
  TkZeroSuppresser& zeroSuppresser() const {return *theTkZeroSuppresser;}
  TkCommonModeCalculator& commonModeCalculator() const {return *theTkCommonModeCalculator;}
  TkPedestalCalculator& pedestalCalculator() const {return *theTkPedestalCalculator;}
  TkNoiseCalculator& noiseCalculator() const {return *theTkNoiseCalculator;}
  TkApvMask& mask() const {return *theTkApvMask;}
  
  //
  // Actions...
  //
  
  virtual RawSignalType rawSignal() = 0;
  virtual SubtractedSignalType pedestalSubtractedSignal() = 0;
  
  virtual SubtractedSignalType CMPSubtractedSignal() = 0;
  
  virtual ZeroSuppressedSignalType zeroSuppressedSignal() = 0;
    
  //
  // Interface to the Reader....
  //
  void setReader(ApvEventReader& in) {theApvEventReader = &in;}
  ApvEventReader* eventReader() const {return theApvEventReader;}
  
  //
  // Give store/load commands to the TkPedestalCalculator and to TkNoiseCalculator, will use ApvEventReader.
  // has to be done here because they have no access to the reader
  //
  virtual void newEvent(){}
  virtual void updateCalibration() = 0;
  virtual bool isCalibrated() = 0;

  

  
 protected:
  TkZeroSuppresser* theTkZeroSuppresser;
  TkCommonModeCalculator* theTkCommonModeCalculator;
  TkPedestalCalculator* theTkPedestalCalculator;
  TkNoiseCalculator* theTkNoiseCalculator;
  TkApvMask* theTkApvMask;
  ApvEventReader* theApvEventReader;  
};
#endif

