#ifndef VIEWER_HistosForAPV
#define VIEWER_HistosForAPV

#include "TH1.h"
#include "TH2.h"
#include "TProfile.h"
#include "ApvAnalysis.h"
#include "TkApvMask.h"
#include <vector>

class HistosForAPV {
public:
   HistosForAPV(int id_apv);
   ~HistosForAPV();

   void  bookHistograms(int itype);
   void  clearHistograms();

   int       getType();
   TH1F*     getRawHisto();
   TProfile* getPed1Histo(); 
   TH1F*     getRNoise1Histo();
   TH1F*     getCMNoise1Histo();
   TH1F*     getCMHisto(); 
   TH1F*     getCalibHisto(int istr); 

   void fillRawHistos(const vector<unsigned short>& RData); 
   void fillHistos(float time, ApvAnalysis::SubtractedSignalType& signal); 
   void fillHistos(const vector<float>& CMode);
   void fillHistos(int flg, ApvAnalysis::SubtractedSignalType& ped, 
                            ApvAnalysis::SubtractedSignalType& noise,
                            TkApvMask::MaskType& mask);

   void saveHistos(int itype);

private: 
   void       initHistograms();

  TH1F*               fRawHisto;
  TH1F*               fInitPed1Histo;
  TH1F*               fInitPed2Histo;
  TH1F*               fInitNoise1Histo;
  TH1F*               fInitNoise2Histo;
  TProfile*           fPed1Histo;
  TH1F*               fPed2Histo;
  TH1F*               fRNoise1Histo;
  TH1F*               fRNoise2Histo;
  TProfile*           fRNoise3Histo;
  TH1F*               fSignal1Histo;
  TH1F*               fSignal2Histo;
  TProfile*           fSignal3Histo;
  TH1F*               fCMNoise1Histo;
  TH1F*               fCMNoise2Histo;
  TProfile*           fCMNoise3Histo;
  TH2F*               fMask1Histo;
  TH1F*               fMask2Histo;
  TH1F*               fCMHisto;
  vector<TH1F*>       fCalibHistoVec;

  bool                fPedHistoBooked;
  bool                fCalibHistoBooked;
  int                 fIdAPV;
};
#endif
