#include "ReferenceApvAnalysis.h"
#include "ApvEventReader.h"
#include "TkNoiseCalculator.h"
#include "TkPedestalCalculator.h"
#include "TkApvMask.h"
#include "TkZeroSuppresser.h"
#include "TkCommonModeCalculator.h"

//
// Really trivial implementation: only return results, do not cache anything...
//

ApvAnalysis::RawSignalType ReferenceApvAnalysis::rawSignal() {
  ApvAnalysis::RawSignalType temp;
  if (theApvEventReader) {
    temp = theApvEventReader->rawSignal(*this);
  }else{
    cerr<< "No Reader defined!" <<endl;
  }
  return temp;
}
ApvAnalysis::SubtractedSignalType ReferenceApvAnalysis::pedestalSubtractedSignal() {
  ApvAnalysis::SubtractedSignalType temp;
  if (theTkPedestalCalculator) {
    temp = theTkPedestalCalculator->doIt(rawSignal());
  }else{
    cerr<< "No PedestalCalculator defined!" <<endl;
  }
  return temp;
}
ApvAnalysis::SubtractedSignalType ReferenceApvAnalysis::CMPSubtractedSignal() {
  ApvAnalysis::SubtractedSignalType temp;
  ApvAnalysis::SubtractedSignalType tempped;
  ApvAnalysis::RawSignalType tempraw;
  if (theTkCommonModeCalculator && theTkPedestalCalculator) {
    tempraw = (*this).rawSignal();
    tempped = theTkPedestalCalculator->doIt(tempraw);
    temp = theTkCommonModeCalculator->doIt(tempped);
  }else{
    cerr<< "No CommonModeCalculator defined!" <<endl;
  }
  return temp;
}
ApvAnalysis::ZeroSuppressedSignalType ReferenceApvAnalysis::zeroSuppressedSignal() {
  ApvAnalysis::ZeroSuppressedSignalType temp;
  if (theTkZeroSuppresser && theTkPedestalCalculator && theTkCommonModeCalculator && theTkNoiseCalculator && theTkApvMask) {
    theTkApvMask->calculateMask(theTkPedestalCalculator->doIt(rawSignal()));
    temp = theTkZeroSuppresser->doIt(theTkCommonModeCalculator->doIt(theTkPedestalCalculator->doIt(rawSignal())), *theTkNoiseCalculator, *theTkApvMask);
  }else{
    cerr<< "Action classes not defined!" <<endl;
  }
  return temp;
}

void ReferenceApvAnalysis::newEvent(){
  pedestalCalculator().newEvent();
  noiseCalculator().newEvent();
}


void ReferenceApvAnalysis::updateCalibration(){
  if (calibrated == true) return;
  ApvAnalysis::SubtractedSignalType dummy = this->pedestalSubtractedSignal();
  if (this->pedestalCalculator().status()->isReady() == true ||this->pedestalCalculator().status()->isUpdating() == true){
    this->noiseCalculator().updateNoise(dummy);
    if (this->noiseCalculator().status()->isReady() == true ||this ->noiseCalculator().status()->isUpdating() == true)
      calibrated = true;
  }
}


