#include "TT6ApvMask.h"
#include <cmath>
#include <numeric>
#include <algorithm>


// 
//  Constructor :
//
TT6ApvMask::TT6ApvMask(int ctype,float ncut,float dcut) :
  theCalculationFlag(ctype),
  theNoiseCut(ncut),
  theDeadCut(dcut)
{
  if (0) cout << "Constructing TT6ApvMask " << endl;
}
// 
//  Destructor :
//
TT6ApvMask::~TT6ApvMask(){
  if (0) cout << "Destructing TT6ApvMask " << endl;
}
//
//  Mask the Raw signal channels 
//
ApvAnalysis::RawSignalType TT6ApvMask::maskRawSignal(ApvAnalysis::RawSignalType in){
  ApvAnalysis::RawSignalType temp;
  for (int i=0; i<in.size(); i++){
    if (theMask[0] == ok) {
      temp.push_back(in[i]);
    }else{
      temp.push_back(0);
    }
  }
  return temp;
}
//
// Mask subtracted signal
//
ApvAnalysis::SubtractedSignalType TT6ApvMask::maskRawSignal(ApvAnalysis::SubtractedSignalType in){
  ApvAnalysis::SubtractedSignalType temp;
  for (int i=0; i<in.size(); i++){
    if (theMask[0] == ok) {
      temp.push_back(in[i]);
    }else{
      temp.push_back(0);
    }
  }
  return temp;
}
//
// Calculate the Mask 
//
void TT6ApvMask::calculateMask(ApvAnalysis::SubtractedSignalType in){
  theMask.clear();

  float sumVal,sqSumVal,avVal,sqAvVal,rmsVal; 
  sumVal   = accumulate( in.begin(), in.end(), 0.0);
  sqSumVal = inner_product( in.begin(), in.end(), in.begin(), 0.0);
  avVal    = (in.size()) ? sumVal/float(in.size()):0.0;
  sqAvVal  = (in.size()) ? sqSumVal/float(in.size()):0.0;
  rmsVal   = sqrt(sqAvVal - avVal*avVal); 
      
  for (int i=0; i<in.size(); i++){
    if (defineNoisy(avVal,rmsVal,in[i])) {
      theMask.push_back(noisy);
    } else if (in[i] < theDeadCut*avVal) {
           theMask.push_back(dead);
    } else {
      theMask.push_back(ok);
    }	  
  }
}
//
//  Identification of Noisy strips (three options available : using cut on 
//   rms of noice distribution, using a percentage cut wrt the average, using
//   a fixed cut 
//
bool TT6ApvMask::defineNoisy(float avrg,float rms,float noise){
  bool temp;
  temp=false;
  if (theCalculationFlag == 1){
   if ((noise-avrg) > theNoiseCut*rms) temp=true;
  } else if (theCalculationFlag == 2){
    if ((noise-avrg) > avrg*theNoiseCut) temp=true;
  } else if (theCalculationFlag == 3){
    if (noise > theNoiseCut) temp=true;
  } 
  return temp;
}
