#include "TT6NoiseCalculator.h"
#include <cmath>
#include <numeric>
#include <algorithm>
//
//  Constructor 
//
TT6NoiseCalculator::TT6NoiseCalculator(int evnt_ini,
                   int evnt_iter, float sig_cut) :
  eventsForInitialization(evnt_ini),
  eventsForIteration(evnt_iter),
  cutToAvoidSignal(sig_cut),
  numberOfEvents(0) ,
  alreadyUsedEvent(false)  
{
  if (0) cout << "Constructing TT6NoiseCalculator " << endl;

  theCMPSubtractedSignal.clear();
  theNoise.clear();
  theNoiseSum.clear();
  theNoiseSqSum.clear();
  theEventPerStrip.clear();
  theStatus.setCalibrating();
}
//
//  Destructor 
//
TT6NoiseCalculator::~TT6NoiseCalculator() {
  if (0) cout << "Destructing TT6NoiseCalculator " << endl;
}
//
// Update the Status of Noise Calculation
//
void TT6NoiseCalculator::updateStatus(){
  if (theStatus.isCalibrating() && numberOfEvents > eventsForInitialization) {
    float sum = accumulate( theNoise.begin(), theNoise.end(), 0. );
    if (sum > 0) {
      theStatus.setUpdating();
    } else {
      theNoise.resize(128,3.);
      theStatus.setUpdating();
      cerr << "*** TT6NoiseCalculator *** WARNING setting default noise value = 3 for all strips *** " << endl;
    }
  }
}

//
// Calculate and update (when needed) Noise Values
//
void TT6NoiseCalculator::updateNoise(ApvAnalysis::SubtractedSignalType in){
  if (alreadyUsedEvent == false) {
    alreadyUsedEvent = true;
    if (theStatus.isCalibrating() && numberOfEvents > eventsForInitialization) updateStatus();
    if (((numberOfEvents-eventsForInitialization)%eventsForIteration) == 1) {

      theNoiseSum.clear();
      theNoiseSqSum.clear();
      theEventPerStrip.clear();
 
      theNoiseSum.resize(in.size(),0.0);
      theNoiseSqSum.resize(in.size(),0.0);
      theEventPerStrip.resize(in.size(),0);
    }    
    unsigned int i;
    if (theStatus.isUpdating()) {
      for (i = 0; i < in.size(); i++) {
	if (fabs(in[i]) < cutToAvoidSignal*theNoise[i]) {
          theNoiseSum[i]   += in[i];
          theNoiseSqSum[i] += in[i]*in[i];
          theEventPerStrip[i]++;
        }
      }
      if (((numberOfEvents-eventsForInitialization) % eventsForIteration) == 0) {

        theCMPSubtractedSignal.clear();
        theNoise.clear();

        for (i = 0; i < in.size(); i++) {
          double avVal   = (theEventPerStrip[i]) 
            ? theNoiseSum[i]/(theEventPerStrip[i]):0.0;
          double sqAvVal = (theEventPerStrip[i]) 
            ? theNoiseSqSum[i]/(theEventPerStrip[i]):0.0;
          double rmsVal  =  sqrt(sqAvVal - avVal*avVal);
      
          theCMPSubtractedSignal.push_back(static_cast<float>(avVal));
          theNoise.push_back(static_cast<float>(rmsVal));
  
	  if (0) cout << " TT6NoiseCalculator::updateNoise " 
                      << theNoiseSum[i] << " " 
	              << theNoiseSqSum[i] << " "
	              << theEventPerStrip[i] << " "  
	              << avVal << " " 
                      << sqAvVal << " " 
                      << (sqAvVal - avVal*avVal) << " " 
                      << rmsVal << endl;
        }
      }
    }
  }
}
//
// Define New Event
// 
void TT6NoiseCalculator::newEvent() {
  alreadyUsedEvent = false;
  numberOfEvents++;
}
