/*
 *  This Window is used to configure Cuts for Bad Channel Definition
 */
#include <iostream.h>
#include <string>

#include "TROOT.h"
#include "TSystem.h"
#include "TKey.h"
#include "TFile.h"
#include "ConfigBadChannel.h"

//
// Constructor  
//
ConfigBadChannel::ConfigBadChannel(const TGWindow *main, UInt_t w, UInt_t h, 
                       int* nchan, int* icut,
		       float* cut_low, float* cut_high, 
                       UInt_t options, ULong_t back) 
             : TGTransientFrame(gClient->GetRoot(), main, w, h, options)
{
  fNAPV    = nchan;
  fCutType = icut;
  fCutLow  = cut_low;
  fCutHigh = cut_high;

  CreateDisplay();
  MapSubwindows();
  Resize(GetDefaultSize());
  SetWindowName("Bad Channel Definition Window");
  SetIconName("Bad Channel Definition Window");

  MapWindow();
  fClient->WaitFor(this);
}
// 
// Destructor 
//
ConfigBadChannel::~ConfigBadChannel() 
{
   delete       fNAPVEntry;
   delete       fCutTypeEntry;
   delete       fPedLowEntry;
   delete       fRNoiLowEntry;
   delete       fCMNoiLowEntry;
   delete       fMaxCalLowEntry;
   delete       fPedHighEntry;
   delete       fRNoiHighEntry;
   delete       fCMNoiHighEntry;
   delete       fMaxCalHighEntry;

   delete       fNAPVFrame;
   delete       fCutTypeFrame;
   delete       fPedFrame;
   delete       fRNoiFrame;
   delete       fCMNoiFrame;
   delete       fMaxCalFrame;

   delete       fDefaultButton;
   delete       fClearButton;
   delete       fSetExitButton;
   delete       fButtonFrame;

   delete       fL;
   delete       fLBut;
          
}
void ConfigBadChannel::CreateDisplay() 
{
  fL = new TGLayoutHints(kLHintsTop|kLHintsLeft, 5, 5, 5, 5);
  fLBut = new TGLayoutHints(kLHintsTop, 20, 20, 20, 20);
  /* Number of APVs in the module */
  fNAPVFrame =  new TGHorizontalFrame(this, 100, 30);
  AddFrame(fNAPVFrame);
  fNAPVEntry = new TGLabeledEntry(fNAPVFrame," # of APVs in Module  : ",70,20,5);
  fNAPVFrame->AddFrame(fNAPVEntry, fL);

  /* Type of the CUT */
  fCutTypeFrame =  new TGHorizontalFrame(this, 100, 30);
  AddFrame(fCutTypeFrame);
  fCutTypeEntry = new TGLabeledEntry(fCutTypeFrame," Type of Cut Definition : ",70,20,5);
  fCutTypeFrame->AddFrame(fCutTypeEntry, fL);

  /* Different Cut Values */

  /* Cuts for Pedestal Values */
  fPedFrame = new TGHorizontalFrame(this, 100, 30);
  AddFrame(fPedFrame);
  fPedLowEntry =  new TGLabeledEntry(fPedFrame," Pedestal ::         Low : ",
                                                               70, 20, 5);
  fPedFrame->AddFrame(fPedLowEntry, fL);
  fPedHighEntry =  new TGLabeledEntry(fPedFrame," High : ",70, 20, 5);
  fPedFrame->AddFrame(fPedHighEntry, fL);

  /* Cuts for Raw Noise Values */
  fRNoiFrame = new TGHorizontalFrame(this, 100, 30);
  AddFrame(fRNoiFrame);
  fRNoiLowEntry =  new TGLabeledEntry(fRNoiFrame," Raw Noise ::       Low : ",
                                                                 70, 20, 5);
  fRNoiFrame->AddFrame(fRNoiLowEntry, fL);
  fRNoiHighEntry =  new TGLabeledEntry(fRNoiFrame," High : ",70, 20, 5);
  fRNoiFrame->AddFrame(fRNoiHighEntry, fL);

  /* Cuts for Common Mode Noise Values */
  fCMNoiFrame = new TGHorizontalFrame(this, 100, 30);
  AddFrame(fCMNoiFrame);
  fCMNoiLowEntry =  new TGLabeledEntry(fCMNoiFrame," CM Noise ::        Low : ",
                                                                    70, 20, 5);
  fCMNoiFrame->AddFrame(fCMNoiLowEntry, fL);
  fCMNoiHighEntry =  new TGLabeledEntry(fCMNoiFrame," High : ",70,20,5);
  fCMNoiFrame->AddFrame(fCMNoiHighEntry, fL);

  /* Cuts for Maximum Calibration Charge Values */
  fMaxCalFrame = new TGHorizontalFrame(this, 100, 30);
  AddFrame(fMaxCalFrame);
  fMaxCalLowEntry =  new TGLabeledEntry(fMaxCalFrame," Max Calibration :: Low : ",
                                                                    70, 20, 5);
  fMaxCalFrame->AddFrame(fMaxCalLowEntry, fL);
  fMaxCalHighEntry =  new TGLabeledEntry(fMaxCalFrame,"High : ",70, 20, 5);
  fMaxCalFrame->AddFrame(fMaxCalHighEntry, fL);

  /* Buttons for different Actions */
  fButtonFrame =  new TGHorizontalFrame(this, 450, 30);
  AddFrame(fButtonFrame);  
  
  fDefaultButton = new TGTextButton(fButtonFrame, "&Default",TB_DEFAULT_VAL);
  fDefaultButton->Resize(100,20);
  fDefaultButton->Associate(this);
  fDefaultButton->SetToolTipText("Set Default Values");
  fButtonFrame->AddFrame(fDefaultButton, fLBut);  

  fClearButton = new TGTextButton(fButtonFrame, "&Clear",TB_CLEAR_VAL);
  fClearButton->Resize(100,20);
  fClearButton->Associate(this);
  fClearButton->SetToolTipText("Clear All Values");
  fButtonFrame->AddFrame(fClearButton, fLBut);  
  
  fSetExitButton = new TGTextButton(fButtonFrame, "&Set and Exit",TB_SET_EXIT);
  fSetExitButton->Resize(300,20);
  fSetExitButton->Associate(this);
  fSetExitButton->SetToolTipText("Set Values and Exit");
  fButtonFrame->AddFrame(fSetExitButton, fLBut);  

  setDefaultValues();
}

// 
// Handle user actions 
//
Bool_t ConfigBadChannel::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2) 
{
    // Handle messages
  switch (GET_MSG(msg)) {

    case kC_COMMAND:
      switch (GET_SUBMSG(msg)) {

        case kCM_BUTTON:
          switch (parm1) {
        
            case TB_DEFAULT_VAL:
              setDefaultValues();
              break;
 
            case TB_CLEAR_VAL:
              clearValues();
              break;

            case TB_SET_EXIT:
              readAndSetValues();
              delete this;
              break;

            default:
              break;
          }
      }
   }
   return kTRUE;
}
// Read Values 
void  ConfigBadChannel::readAndSetValues() {
  *fNAPV        = atoi(fNAPVEntry->GetText());
  *fCutType     = atoi(fCutTypeEntry->GetText());
 
  *(fCutLow+0)  = atof(fPedLowEntry->GetText());
  *(fCutLow+1)  = atof(fRNoiLowEntry->GetText());
  *(fCutLow+2)  = atof(fCMNoiLowEntry->GetText());
  *(fCutLow+3)  = atof(fMaxCalLowEntry->GetText());

  *(fCutHigh+0) = atof(fPedHighEntry->GetText());
  *(fCutHigh+1) = atof(fRNoiHighEntry->GetText());
  *(fCutHigh+2) = atof(fCMNoiHighEntry->GetText());
  *(fCutHigh+3) = atof(fMaxCalHighEntry->GetText());
}
// Clear Values 
void  ConfigBadChannel::clearValues() {
  fNAPVEntry->SetText(" ");
  fCutTypeEntry->SetText(" ");   
                                  
  fPedLowEntry->SetText(" ");    
  fRNoiLowEntry->SetText(" ");   
  fCMNoiLowEntry->SetText(" ");  
  fMaxCalLowEntry->SetText(" "); 
                                  
  fPedHighEntry->SetText(" ");   
  fRNoiHighEntry->SetText(" ");  
  fCMNoiHighEntry->SetText(" "); 
  fMaxCalHighEntry->SetText(" ");

  *fNAPV        = 0;
  *fCutType     = 0;
 
  *(fCutLow+0)  = 0.0;
  *(fCutLow+1)  = 0.0;
  *(fCutLow+2)  = 0.0;
  *(fCutLow+3)  = 0.0;

  *(fCutHigh+0) = 0.0;
  *(fCutHigh+1) = 0.0;
  *(fCutHigh+2) = 0.0;
  *(fCutHigh+3) = 0.0;
}
// Set Default Values
void  ConfigBadChannel::setDefaultValues() {

  fNAPVEntry->SetText("4");
  fCutTypeEntry->SetText("1");   
                                  
  fPedLowEntry->SetText("0.3");    
  fRNoiLowEntry->SetText("0.3");   
  fCMNoiLowEntry->SetText("0.3");  
  fMaxCalLowEntry->SetText("0.3"); 
                                  
  fPedHighEntry->SetText("0.2");   
  fRNoiHighEntry->SetText("0.2");  
  fCMNoiHighEntry->SetText("0.2"); 
  fMaxCalHighEntry->SetText("0.2");
 
  readAndSetValues();
}
