//
//  This Window is used to configure Infor for XML File
//
#include <iostream.h>
#include <string>

#include "TROOT.h"
#include "TSystem.h"
#include "TKey.h"
#include "TGFileDialog.h"
#include "TGMsgBox.h"

#include "ModuleTestApp.h"
#include "ConfigXMLInfo.h"
//
// Constructor  
//
ConfigXMLInfo::ConfigXMLInfo(const TGWindow *main, UInt_t w, UInt_t h, 
                       int* idmod, int* icut,
                       UInt_t options, ULong_t back) 
             : TGTransientFrame(gClient->GetRoot(), main, w, h, options)
{
  fParent     = main;

  fIDModule   = idmod;
  fCutFlag    = icut;

  CreateDisplay();
  MapSubwindows();
  Resize(GetDefaultSize());
  SetWindowName("XML File Definition Window");
  SetIconName("XML File Definition Window");

  MapWindow();
  fClient->WaitFor(this);
}
// 
// Destructor 
//
ConfigXMLInfo::~ConfigXMLInfo() 
{
   delete   fIDModuleLab;
   delete   fIDModuleNE;
   delete   fIDModuleFrame;

   delete   fRootFileLab;
   delete   fRootFileEntry;   
   delete   fRootFileBut;
   delete   fRootFileFrame;

   delete   fBadChanDefLab;
   delete   fBadChanDefRB[0];
   delete   fBadChanDefRB[1];
   delete   fBadChanDefFrame;

   delete   fOperatorLab;
   delete   fOperatorEntry;   
   delete   fOperatorFrame;

   delete   fDefaultButton;
   delete   fClearButton;
   delete   fCancelButton;
   delete   fSetExitButton;
   delete   fButtonFrame;

   delete   fL;
   delete   fLBut;
}
void ConfigXMLInfo::CreateDisplay() 
{
  fL = new TGLayoutHints(kLHintsTop|kLHintsLeft, 5, 5, 5, 5);
  fLBut = new TGLayoutHints(kLHintsTop, 20, 20, 20, 20);

  // Module ID and # of Chips in the Module
  fIDModuleFrame =  new TGHorizontalFrame(this, 100, 30);
  AddFrame(fIDModuleFrame);

  fIDModuleLab = new TGLabel(fIDModuleFrame, "ID # for Module : ");
  fIDModuleFrame->AddFrame(fIDModuleLab, fL); 
  fIDModuleNE = new TGNumberEntry(fIDModuleFrame, *fIDModule, 5, 
   NE_ID_MOD, TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative);
  fIDModuleNE->Associate(this);
  fIDModuleFrame->AddFrame(fIDModuleNE, fL);

  // Root File Name
  fRootFileFrame =  new TGHorizontalFrame(this, 100, 30);
  AddFrame(fRootFileFrame);

  fRootFileLab = new TGLabel(fRootFileFrame, "Root File Name : ");
  fRootFileFrame->AddFrame(fRootFileLab, fL);
 
  fRootFileEntry = new TGTextEntry(fRootFileFrame, " ", TE_ROOT_FILE);
  fRootFileFrame->AddFrame(fRootFileEntry, fL);
  fRootFileEntry->Resize(150, 25);
  fRootFileEntry->Associate(this);

  fRootFileBut = new TGTextButton(fRootFileFrame,"&Browse",TB_BROWSE_RFILE);
  fRootFileBut->Associate(this);
  fRootFileBut->SetToolTipText("Browse files");
  fRootFileFrame->AddFrame(fRootFileBut,fL);

  // Type of the CUT 
  fBadChanDefFrame =  new TGHorizontalFrame(this, 100, 30);
  AddFrame(fBadChanDefFrame);
  fBadChanDefLab = new TGLabel(fBadChanDefFrame, "Bad Channel Definition: ");
  fBadChanDefFrame->AddFrame(fBadChanDefLab, fL);  
  fBadChanDefRB[0] = new TGRadioButton(fBadChanDefFrame, 
				  new TGHotString(" Default "),RB_DEF_CUT);  
  fBadChanDefRB[1] = new TGRadioButton(fBadChanDefFrame, 
				     new TGHotString(" Select "),RB_SEL_CUT);
  for (int k = 0; k < 2; k++) { 
    fBadChanDefRB[k]->Associate(this);
    fBadChanDefFrame->AddFrame(fBadChanDefRB[k], fL);
  }

  // Operator Name
  fOperatorFrame =  new TGHorizontalFrame(this, 100, 30);
  AddFrame(fOperatorFrame);

  fOperatorLab = new TGLabel(fOperatorFrame, "Operator Name : ");
  fOperatorFrame->AddFrame(fOperatorLab, fL);
 
  fOperatorEntry = new TGTextEntry(fOperatorFrame, " ", TE_OPR_NAME);
  fOperatorEntry->Resize(150, 25);
  fOperatorEntry->Associate(this);
  fOperatorFrame->AddFrame(fOperatorEntry, fL);

  /* Buttons for different Actions */
  fButtonFrame =  new TGHorizontalFrame(this, 450, 30);
  AddFrame(fButtonFrame);  
  
  fDefaultButton = new TGTextButton(fButtonFrame, "&Default",TB_DEF_ALL);
  fDefaultButton->Resize(100,20);
  fDefaultButton->Associate(this);
  fDefaultButton->SetToolTipText("Set Default Values");
  fButtonFrame->AddFrame(fDefaultButton, fLBut);  

  fClearButton = new TGTextButton(fButtonFrame, "&Clear",TB_CLEAR_ALL);
  fClearButton->Resize(100,20);
  fClearButton->Associate(this);
  fClearButton->SetToolTipText("Clear All Values");
  fButtonFrame->AddFrame(fClearButton, fLBut);  
  
  fCancelButton = new TGTextButton(fButtonFrame, "&Cancel",TB_CANCEL_CONF);
  fCancelButton->Resize(100,20);
  fCancelButton->Associate(this);
  fCancelButton->SetToolTipText("Cancel Action");
  fButtonFrame->AddFrame(fCancelButton, fLBut);  

  fSetExitButton = new TGTextButton(fButtonFrame, "&Set and Exit",TB_EXIT_CONF);
  fSetExitButton->Resize(300,20);
  fSetExitButton->Associate(this);
  fSetExitButton->SetToolTipText("Set Values and Exit");
  fButtonFrame->AddFrame(fSetExitButton, fLBut);  

  setDefaultValues();
}

// 
// Handle user actions 
//
Bool_t ConfigXMLInfo::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2) 
{
    // Handle messages
  switch (GET_MSG(msg)) {

    case kC_COMMAND:
      switch (GET_SUBMSG(msg)) {

        case kCM_RADIOBUTTON:
          switch (parm1) {

	  case RB_DEF_CUT:
	    {
              fBadChanDefRB[1]->SetState(kButtonUp);
              *fCutFlag = 0;
              break;
            }

          case RB_SEL_CUT:
	    {
              fBadChanDefRB[0]->SetState(kButtonUp);
              *fCutFlag = 1;
              break;
            }          
          }
        case kCM_BUTTON:
          switch (parm1) {
        
           case TB_DEF_ALL:
             setDefaultValues();
             break;
 
           case TB_CLEAR_ALL:
             clearValues();
             break;

           case TB_CANCEL_CONF:
             delete this;
             break;

           case TB_EXIT_CONF:
             readAndSetValues();
             delete this;
             break;

	  case TB_BROWSE_RFILE:
             selectRootFile();
             break;

            default:
            break;
          }
      }
   }
   return kTRUE;
}
// Read Values 
void  ConfigXMLInfo::readAndSetValues() {
  *fIDModule    = fIDModuleNE->GetIntNumber();
  if (fBadChanDefRB[0]->GetState() == kButtonDown) *fCutFlag = 0;
  else if (fBadChanDefRB[1]->GetState() == kButtonDown) *fCutFlag = 1;  
  fRootFileName = fRootFileEntry->GetText();
  fOperatorName = fOperatorEntry->GetText();
  cout << " File : " << fRootFileName << " Operator : " 
       << fOperatorName << " Module ID " << *fIDModule 
       << " Cut Flag " << *fCutFlag << endl;
  ((ModuleTestApp*) fParent)->setLoclDBFile(fRootFileName);
  ((ModuleTestApp*) fParent)->setOperatorName(fOperatorName);
}
// Clear Values 
void  ConfigXMLInfo::clearValues() {
  fIDModuleNE->SetIntNumber(0);
  fRootFileEntry->SetText("");
  fOperatorEntry->SetText("");
}
// Set Default Values
void  ConfigXMLInfo::setDefaultValues() {
  fIDModuleNE->SetIntNumber(100);
  fBadChanDefRB[0]->SetState(kButtonDown);
  fRootFileEntry->SetText("");
  fOperatorEntry->SetText("");
}
// Browse and Select Root File
void ConfigXMLInfo::selectRootFile() {
  TGFileInfo fi;  
  const char *fRootFileTypes[] = {"ROOT files",    "*.root"};
  fi.fFileTypes = fRootFileTypes;
  new TGFileDialog(fClient->GetRoot(), this, kFDOpen, &fi);
  if (!fi.fFilename) {
    int retval;
    new TGMsgBox(fClient->GetRoot(), this, "XML Database Message",
    "File with specified name does not exist !!",
     kMBIconExclamation, kMBApply, &retval);  
    return;
  } else {
    fRootFileEntry->SetText(fi.fFilename);
  }
}
