//
//  A Root based GUI with embedded canvas to draw histogram.
//  At present designed to be controlled completely by the
//  Root MainLoop. 
//
//
#include <iostream.h>
#include <fstream.h>  // controls the class: ofstream
#include <strstream.h>
#include <string>
#include <map>

#include "TROOT.h" 
#include "TApplication.h"
#include "TVirtualX.h"
#include "TSystem.h"
#include "TEnv.h"
#include "TCanvas.h"
#include "TStyle.h"

#include "TGMsgBox.h"
#include "TFile.h"
#include "TGFileDialog.h"
#include "TGTableLayout.h"
#include "TextViewer.h"

#include "Histogrammer.h" 
#include "ConfigBadChannel.h"
#include "ConfigAutoRunInfo.h"
#include "ModuleTestApp.h"

//
// Constructor of the class ModuleTestApp, which consists of a Menu, an 
// embeddedcanvas A button panel and a status area. The constructor 
// is responsible for drawing a panel and mapping all subpanels inside
// 
ModuleTestApp::ModuleTestApp(const TGWindow *p, UInt_t w, UInt_t h, 
                 UInt_t options = kMainFrame|kVerticalFrame) :
                 TGMainFrame(p, w, h, options) 
{

  addTabs();                           // Add 'Display' and 'Import' tabs 
  CreateDisplay();
  MapSubwindows();                     // Make the subwindows visible
  SetWindowName("Module Test Client");   // Set window title 
  Resize(GetDefaultSize());            // Set to default window size
  MapWindow();                         // Make the application visible
  fRawTimer     = new TTimer(100);
  fPedTimer     = new TTimer(100);
  fRNoiseTimer  = new TTimer(100);
  fCMNoiseTimer = new TTimer(100);
  fCMTimer      = new TTimer(100);
  fCalibTimer   = new TTimer(100);

  fRawTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getRawCanvas(), "Modified()");
  fRawTimer->Connect("Timeout()", "TCanvas", 
                 fOnlineHistogramTab->getRawCanvas(), "Update()");
  fPedTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getPedCanvas(), "Modified()");
  fPedTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getPedCanvas(), "Update()");
  fRNoiseTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getRNoiseCanvas(), "Modified()");
  fRNoiseTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getRNoiseCanvas(), "Update()");
  fCMNoiseTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getCMNoiseCanvas(), "Modified()");
  fCMNoiseTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getCMNoiseCanvas(), "Update()");
  fCMTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getCMCanvas(), "Modified()");
  fCMTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getCMCanvas(), "Update()");
  fCalibTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getCalibCanvas(), "Modified()");
  fCalibTimer->Connect("Timeout()", "TCanvas", 
                  fOnlineHistogramTab->getCalibCanvas(), "Update()");
}

// Destructor  
ModuleTestApp::~ModuleTestApp() 
{
  
  delete fRawTimer;
  delete fPedTimer;
  delete fRNoiseTimer;
  delete fCMNoiseTimer;
  delete fCMTimer;
  delete fCalibTimer;

  delete fSaveConfBut;
  delete fSaveXMLBut;
  delete fAutoStartBut;
  delete fReleaseBut;
  delete fHelpBut;
  delete fReleaseExitBut;
  delete fButtonFrame;
  delete fL;

  delete fMainTabFrame;
  delete fMainTab;
  delete tf; 
  delete fHVSettingTab;
  delete fControlSettingTab;
  delete fOnlineHistogramTab;
  delete fHistogramViewingTab;
  delete fGeneralLayout;

  Histogrammer* fHistogrammer = Histogrammer::Instance();
  fHistogrammer->clearAllHistograms();
}
void ModuleTestApp::CreateDisplay() 
{
  fL     = new TGLayoutHints(kLHintsNormal,15, 15, 5, 5);
  fButtonFrame = new TGCompositeFrame(this, 10, 10);
  fButtonFrame->SetLayoutManager(new TGMatrixLayout(fButtonFrame, 0, 3, 6));
  AddFrame(fButtonFrame, fL);

  // Save Configuration Button   
  fSaveConfBut = new TGTextButton(fButtonFrame, " &Save Configuration ", 
                                                    TB_MAIN_CONF_SAVE);
  fSaveConfBut->Resize(160,20);
  fSaveConfBut->Associate(this);
  fSaveConfBut->SetToolTipText("Save Configuration file in xml format");
  fButtonFrame->AddFrame(fSaveConfBut,fL);

  // Save to Xml File
  fSaveXMLBut = new TGTextButton(fButtonFrame, "&Save Root File to XML",
                                                     TB_MAIN_XML_SAVE);
  fSaveXMLBut->Resize(160,20);
  fSaveXMLBut->Associate(this);
  fSaveXMLBut->SetToolTipText("Save Data to XML File for Tracker DB");
  fButtonFrame->AddFrame(fSaveXMLBut, fL);

  // Automatic Start Up DAQ from Scenario File
  fAutoStartBut = new TGTextButton(fButtonFrame, "&Automatic DAQ Start",
                                                  TB_MAIN_AUTO_START);
  fAutoStartBut->Resize(160,20);
  fAutoStartBut->Associate(this);
  fAutoStartBut->SetToolTipText("Automatic Start Up DAQ from Scenario File");
  fButtonFrame->AddFrame(fAutoStartBut, fL);

  // Release Channel  Button   
  fReleaseBut = new TGTextButton(fButtonFrame, " &Release Channels ", 
                                                     TB_MAIN_REL);
  fReleaseBut->Resize(160,20);
  fReleaseBut->Associate(this);
  fReleaseBut->SetToolTipText("Release All Channels");
  fButtonFrame->AddFrame(fReleaseBut,fL);

  // Help  Button   
  fHelpBut = new TGTextButton(fButtonFrame, " &Online Help ", 
                                                TB_MAIN_HELP);
  fHelpBut->Resize(160,20);
  fHelpBut->Associate(this);
  fHelpBut->SetToolTipText("Online Help ");
  fButtonFrame->AddFrame(fHelpBut,fL);

  // Release Channels and Quit Button   
  fReleaseExitBut = new TGTextButton(fButtonFrame, 
                         " &Quit and Release Channels ", TB_MAIN_RELX);
  fReleaseExitBut->Resize(160,20);
  fReleaseExitBut->Associate(this);
  fReleaseExitBut->SetToolTipText("Release All Channels and Quit");
  fButtonFrame->AddFrame(fReleaseExitBut,fL);
}
// 
// Add the tab widget with 3 tab elements below the menubar
// The 'Display' tab has an embedded canvas which draws histograms,
// an active list view to show a list (of histograms) which can be 
// single clicked to display the corresponding histogram; and a 
// panel of button to view histograms. 
// 
void ModuleTestApp::addTabs() 
{
  fGeneralLayout = new TGLayoutHints(kLHintsLeft|kLHintsTop|kLHintsExpandX,1,1,1,1);
  // Create a composite frame which holds the tab and add to the mainframe 
  fMainTabFrame = new TGCompositeFrame(this, 10, 10, 
        kHorizontalFrame | kSunkenFrame);
  AddFrame(fMainTabFrame, fGeneralLayout);

  // Create a Tab widget and add to the composite frame  
  fMainTab = new TGTab(fMainTabFrame, 10, 10);
  fMainTabFrame->AddFrame(fMainTab, fGeneralLayout);
  fMainTab->Associate(this);

  // Now add individual tabs to the tab widgets 


  // Control Setting Tab 
   tf = fMainTab->AddTab("Control");
   fControlSettingTab = new ControlSettingTab(this, tf, 20, 20, kVerticalFrame);
   tf->AddFrame(fControlSettingTab, 
    new TGLayoutHints(kLHintsLeft|kLHintsTop,1,1,1,0));


  // Online Histogram Plotting Tab 
   tf = fMainTab->AddTab("Online");
   fOnlineHistogramTab = new OnlineHistogramTab(this, tf, 20, 20, kVerticalFrame);
   tf->AddFrame(fOnlineHistogramTab, 
    new TGLayoutHints(kLHintsLeft|kLHintsTop,1,1,1,0));

  // Histogram Analysis  Tab 
   tf = fMainTab->AddTab("Analysis");
   fHistogramViewingTab = new HistogramViewingTab(this, tf, 20, 20, kVerticalFrame);
   tf->AddFrame(fHistogramViewingTab, 
    new TGLayoutHints(kLHintsLeft|kLHintsTop,1,1,1,0));

  // High Voltage Setting Tab 
  tf = fMainTab->AddTab("HV");
  fHVSettingTab = new HVSettingTab(this, tf, 20, 20, kVerticalFrame);
  tf->AddFrame(fHVSettingTab, 
    new TGLayoutHints(kLHintsLeft|kLHintsTop,1,1,1,0));
}
//
// Close Window
//
void ModuleTestApp::CloseWindow()
{
   TGMainFrame::CloseWindow();
   gApplication->Terminate(0);
}
// 
// Handle user actions 
//
Bool_t ModuleTestApp::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2) 
{
    // Handle messages
  switch (GET_MSG(msg)) {
    case kC_COMMAND:
      switch (GET_SUBMSG(msg)) {
        case kCM_BUTTON:
          switch (parm1) {

            case TB_MAIN_CONF_SAVE:
              saveConfiguration();
              break;

            case TB_MAIN_XML_SAVE:
              doItXMLDBase();
              break;

            case TB_MAIN_AUTO_START:
              doItAutoRun();
              break;

            case TB_MAIN_REL:
              releaseChannels();
              break;

            case TB_MAIN_HELP:
              showHelp();
              break;

            case TB_MAIN_RELX:
              releaseAndExit();
              break;

            default:
              break;
          }

        case kCM_TAB:
	  if (parm1 == 1) {
            if (fControlSettingTab->getState() == 1) {
              fRawTimer->Start();
              if (fControlSettingTab->getProcessType() > 1) {
	        fPedTimer->Start();
	        fRNoiseTimer->Start();
	        fCMNoiseTimer->Start();
	        fCMTimer->Start();
              }
              if (fControlSettingTab->getProcessType() == 3) {
                fCalibTimer->Start();
              }
            }
	  } else {
            if (fControlSettingTab->getState() == 1) {
	      fRawTimer->Stop();
              if (fControlSettingTab->getProcessType() > 1) {
	        fPedTimer->Stop();
	        fRNoiseTimer->Stop();
	        fCMNoiseTimer->Stop();
	        fCMTimer->Stop();
              }
              if (fControlSettingTab->getProcessType() == 3) {
                fCalibTimer->Stop();
              }
            }
          }
          break;
    
        default:
        break;

      }
    default:
    break;
  }
  return kTRUE;
}
//
// Save Configuration in xml file
//
void ModuleTestApp::saveConfiguration()
{
  cout << " Not Yet Implemented " << endl;
}
//
// Save Module Informations in the XML File For the Tracker Data Base
//
void ModuleTestApp::doItXMLDBase(){
  cout << " Not Yet Implemented " << endl;
}
//
// Release Channels
//
void ModuleTestApp::releaseChannels() {
  cout << " Release All Channels " << endl;
}
//
// Release Channels and Exit from the application
//
void ModuleTestApp::releaseAndExit()
{
  cout << " Release Channels and Exit from the Application" << endl;
  releaseChannels();
  CloseWindow();
}
//
// Get High Voltage Setting Tab
//
inline HVSettingTab* ModuleTestApp::getHVSettingTab() {
  return fHVSettingTab;
}
//
// Get Control Setting Tab
//
inline ControlSettingTab* ModuleTestApp::getControlSettingTab() {
  return fControlSettingTab;
}
//
// Get Histogram Setting Tab
//
inline HistogramViewingTab* ModuleTestApp::getHistogramViewingTab() {
  return fHistogramViewingTab;
}
//
// Get Online Histogram Tab
//
inline OnlineHistogramTab* ModuleTestApp::getOnlineHistogramTab() {
  return fOnlineHistogramTab;
}
//
// Get Current Tab ID Number
//
inline int ModuleTestApp::getCurrentTabId() {
  int fId = fMainTab->GetCurrent();
  return fId;
}
//
// Show Help Corresponding to TAB ID 
//
void ModuleTestApp::showHelp() {
  string filename;
  string tabName(fMainTab->GetCurrentTab()->GetString());

  if (0) cout << "Tabname = " << tabName << endl;
  else if (tabName == "Control") {
    filename = "./doc/txt/Control.txt"; 
  } 
  else if (tabName == "Online") {
    filename = "./doc/txt/Online.txt"; 
  } 
  else if (tabName == "Analysis") {
    filename = "./doc/txt/Analysis.txt" ; 
  } 
  else if (tabName == "HV") {
    filename = "./doc/txt/HV.txt"; 
  } 
  else {
    filename = "./doc/txt/OverAll.txt";
  }
  TextViewer *tv = new TextViewer(this, 600, 400);
  tv->LoadFile(filename.c_str());
  tv->Popup();
}
//
// -- Set Local DB Root File Name 
//
void ModuleTestApp::setLoclDBFile(string fname) {
  fLocalDBFile = fname;
}
//
// -- Set Operator Name
//
void ModuleTestApp::setOperatorName(string oname) {
  fOperatorName = oname;
}
//
// -- Set Configuration File Name for Automatic Data Taking
//
void ModuleTestApp::setRunConfigFile(string fname) {
  fRunConfigFileName = fname;
}
//
// -- Set File Name for storing Root Histograms
//
void ModuleTestApp::setSaveRootFile(string fname) {
  fSaveRootFileName = fname;
}
//
// -- Get File Name for storing Root Histograms
//
inline string ModuleTestApp::getSaveRootFile() {
  return fSaveRootFileName;
}
//
// Get TGTab Widget
//
inline TGTab* ModuleTestApp::getTab() {
  return fMainTab;
}
// Get Root File Name Containing Histograms
void ModuleTestApp::doItAutoRun(){
  cout << " Not Yet Implemented " << endl;
}
#define __MAIN__
#ifdef __MAIN__
// Main program 

extern void InitGui(); 
VoidFuncPtr_t initfuncs[] = { InitGui, 0 };
TROOT root("GUI", "GUI test environement");
int main(int argc, char **argv)
{
   TApplication theApp("App", &argc, argv);

   if (gROOT->IsBatch()) {
      fprintf(stderr, "%s: cannot run in batch mode\n", argv[0]);
      return 1;
   }

   ModuleTestApp mainWindow(gClient->GetRoot(), 800, 520);
   gStyle->SetOptStat(1);
   theApp.Run();
   return 0;
}
#endif
