/*  Online Histogram plotting Tab */
#include <iostream.h>
#include <fstream.h>  // controls the class: ofstream
#include <strstream.h>
#include "TCanvas.h"
#include "ModuleTestApp.h"
#include "Histogrammer.h"
#include "OnlineHistogramTab.h"
//
// Constructor  
//
OnlineHistogramTab::OnlineHistogramTab(const TGMainFrame *parent, const TGWindow* main, 
              UInt_t w, UInt_t h, UInt_t options, ULong_t back) 
            : TGCompositeFrame(main, w, h, options, back)
{
  fParent = parent;
  fChannelNumber = 0;
  CreateDisplay();
}
// 
// Destructor 
//
OnlineHistogramTab::~OnlineHistogramTab() 
{    
   delete  fChannelLabel;
   delete  fStripLabel;
   delete  fChannelEntry;
   delete  fStripEntry;
   delete  fChannelStripFrame;

   delete  fRawCanvas;
   delete  fPedCanvas;
   delete  fRNoiseCanvas;
   delete  fCMNoiseCanvas;
   delete  fCMCanvas;
   delete  fCalibCanvas;

   delete  fRawFrame; 
   delete  fPedFrame; 
   delete  fRNoiseFrame;
   delete  fCMNoiseFrame;
   delete  fCMFrame;
   delete  fCalibFrame;

   delete  fOnlinePlotTab;
   delete  fOnlinePlotTabFrame;
   delete  fL1;
   delete  fL2;   
}
void OnlineHistogramTab::CreateDisplay() 
{
  fL1 = new TGLayoutHints(kLHintsLeft|kLHintsTop|kLHintsExpandX,5,5,5,5);
  fL2 = new TGLayoutHints(kLHintsNormal,15,15,10,10);
  /* Create a composite frame which holds the tab and add to the mainframe */
  fOnlinePlotTabFrame = new TGCompositeFrame(this,10,10,kVerticalFrame|kSunkenFrame);
  AddFrame(fOnlinePlotTabFrame, fL1);

  fChannelStripFrame = new TGHorizontalFrame(fOnlinePlotTabFrame, 300, 30);
  fOnlinePlotTabFrame->AddFrame(fChannelStripFrame);

  /* Add Channel Counter Widget */  
  fChannelLabel = new TGLabel(fChannelStripFrame, " APV Number ");
  fChannelStripFrame->AddFrame(fChannelLabel,fL2);

  fChannelEntry = new TGNumberEntry(fChannelStripFrame, 1, 3, NE_CHAN, 
        TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative,
                                    TGNumberFormat::kNELLimitMinMax,1,6);
  fChannelEntry->Associate(this);
  fChannelStripFrame->AddFrame(fChannelEntry, fL2);     

  /* Add Strip Counter Widget */  
  fStripLabel = new TGLabel(fChannelStripFrame, " Strip Number ");
  fChannelStripFrame->AddFrame(fStripLabel,fL2);

  fStripEntry = new TGNumberEntry(fChannelStripFrame, 1, 3, NE_STRIP, 
        TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative,
                                    TGNumberFormat::kNELLimitMinMax,1,128);
  fStripEntry->Associate(this);
  fChannelStripFrame->AddFrame(fStripEntry, fL2);
  fStripEntry->SetState(kFALSE);     

/* Create a Tab widget and add to the composite frame */ 
  fOnlinePlotTab = new TGTab(fOnlinePlotTabFrame, 10, 10);
  fOnlinePlotTab->Associate(this);
  fOnlinePlotTabFrame->AddFrame(fOnlinePlotTab, fL1);

  fRawFrame     = fOnlinePlotTab->AddTab("Raw Data");
  fPedFrame     = fOnlinePlotTab->AddTab("Pedestal");
  fRNoiseFrame  = fOnlinePlotTab->AddTab("Raw Noise");
  fCMNoiseFrame = fOnlinePlotTab->AddTab("CM Noise");
  fCMFrame      = fOnlinePlotTab->AddTab("Common Mode");
  fCalibFrame   = fOnlinePlotTab->AddTab("Calibration"); 

  /* Raw Data Plotting */
  fRawCanvas = new TRootEmbeddedCanvas("Raw Data Canvas",
                                 fRawFrame, 450, 450);
  fRawFrame->AddFrame(fRawCanvas, fL1);
  /* Pedestal Plotting */
  fPedCanvas = new TRootEmbeddedCanvas("Pedestal Canvas", 
                        fPedFrame, 450, 450);
  fPedFrame->AddFrame(fPedCanvas, fL1);
  /* Raw Noise Plotting */
  fRNoiseCanvas = new TRootEmbeddedCanvas("Raw Noise Canvas", 
                        fRNoiseFrame, 450, 450);
  fRNoiseFrame->AddFrame(fRNoiseCanvas, fL1);
  /* Common Mode Subtracted Noise Plotting */
  fCMNoiseCanvas = new TRootEmbeddedCanvas("CM Noise Canvas", 
                        fCMNoiseFrame, 450, 450);
  fCMNoiseFrame->AddFrame(fCMNoiseCanvas, fL1);
  /* Common Mode Plotting */
  fCMCanvas = new TRootEmbeddedCanvas("Common Mode Canvas", 
                        fCMFrame, 450, 450);
  fCMFrame->AddFrame(fCMCanvas, fL1);
  /* Calibration  Signal Plotting */
  fCalibCanvas = new TRootEmbeddedCanvas("Calibration Canvas", 
                        fCalibFrame, 450, 450);
  fCalibFrame->AddFrame(fCalibCanvas, fL1);

}
// 
// Handle user actions 
//
Bool_t OnlineHistogramTab::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2) 
{
    // Handle messages
  switch (GET_MSG(msg)) {
    case kC_TEXTENTRY:
      switch (GET_SUBMSG(msg)) {
        
        case kTE_TEXTCHANGED: 
          if (parm1 == NE_CHAN || parm1 == NE_STRIP) {
            int ichan = getAPVIdToBePlotted();
            int istr  = getStripIdToBePlotted();
            ((ModuleTestApp *)fParent)->getControlSettingTab()->addHistoToCanvas(ichan, istr);
          }
      }
    case kC_COMMAND:
      switch (GET_SUBMSG(msg)) {

        case kCM_BUTTON:
          switch (parm1) {

            default:
              break;
          }
        case kCM_TAB:
          {
            if (parm1 == 5) {
              fStripEntry->SetState(kTRUE); 
            } else {
              fStripEntry->SetState(kFALSE);
            } 
            break;
	  }

        default:
        break;
      }
    default:
    break;
   }
   return kTRUE;
}
//
// -- Get Current Tab ID Number
//
inline int OnlineHistogramTab::getCurrentTabId() {
  int fId = fOnlinePlotTab->GetCurrent();
  return fId;
}
//
// -- Get Raw Data Plotting Canvas
//
inline TCanvas* OnlineHistogramTab::getRawCanvas() {
  TCanvas* fTCanvas = fRawCanvas->GetCanvas();
  return fTCanvas;
}
//
// -- Get Pedestal Plotting Canvas
//
inline TCanvas* OnlineHistogramTab::getPedCanvas() {
  TCanvas* fTCanvas = fPedCanvas->GetCanvas();
  return fTCanvas;
}
//
// -- Get Raw Noise Plotting Canvas
//
inline TCanvas* OnlineHistogramTab::getRNoiseCanvas() {
  TCanvas* fTCanvas = fRNoiseCanvas->GetCanvas();
  return fTCanvas;
}
//
// -- Get Common Mode Subtracted Noise Plotting Canvas
//
inline TCanvas* OnlineHistogramTab::getCMNoiseCanvas() {
  TCanvas* fTCanvas = fCMNoiseCanvas->GetCanvas();
  return fTCanvas;
}
//
// -- Get Common Mode Plotting Canvas
//
inline TCanvas* OnlineHistogramTab::getCMCanvas() {
  TCanvas* fTCanvas = fCMCanvas->GetCanvas();
  return fTCanvas;
}
//
// -- Get Calibration pulse shape Plotting Canvas
//
inline TCanvas* OnlineHistogramTab::getCalibCanvas() {
  TCanvas* fTCanvas = fCalibCanvas->GetCanvas();
  return fTCanvas;
}
//
// -- Get APV ID to be plotted
//
inline int  OnlineHistogramTab::getAPVIdToBePlotted() {
  int idAPV = fChannelEntry->GetIntNumber();
  return idAPV;
}
//
// -- Get Strip ID to be plotted
//
inline int  OnlineHistogramTab::getStripIdToBePlotted() {
  int idStrip = fStripEntry->GetIntNumber();
  return idStrip;
}
//
// -- Get APV ID to be plotted
//
inline TCanvas* OnlineHistogramTab::getSelectedCanvas() {
  int fId = fOnlinePlotTab->GetCurrent();
  TCanvas* fTCanvas;
  if (fId == 0) fTCanvas = fRawCanvas->GetCanvas();
  if (fId == 1) fTCanvas = fPedCanvas->GetCanvas();
  if (fId == 2) fTCanvas = fRNoiseCanvas->GetCanvas();
  if (fId == 3) fTCanvas = fCMNoiseCanvas->GetCanvas();
  if (fId == 4) fTCanvas = fCMCanvas->GetCanvas();
  if (fId == 5) fTCanvas = fCalibCanvas->GetCanvas();
  return fTCanvas;
}

