#include <iostream.h>
#include <string.h>

#include "TGMsgBox.h"
#include "TGFileDialog.h"

#include "PrintSetup.h"

static const char *printFileTypes[] = {"PS Files",     "*.ps",
				       "EPS Files",    "*.eps"};

/*
 *  Constructor. 
 */

PrintSetup::PrintSetup(const TGWindow *p, const TGWindow *main, 
              UInt_t w, UInt_t h, Int_t *ret_code, Int_t *flag, char *pname,
              UInt_t options): TGTransientFrame(p, main, w, h, options)
{
  fParent = main;
  fRetCode = ret_code;
  fFlagVal = flag;
  fOutputName = pname;
  
  CreateDisplay();
  MapSubwindows();
  Resize(GetDefaultSize());
  SetWindowName("Print Dialog");

  MapWindow();
  fClient->WaitFor(this);
}

/*
 * Destructor
 */
PrintSetup::~PrintSetup() 
{
   // Buttons
   delete fPrintButton; 
   delete fCancelButton;

   delete fL1;
   delete fL2;


   // Print Destination
   delete fDestRB[0];
   delete fDestRB[1];

   delete fPrinterEntry;  // The buffers associated are deleted automatically 
   delete fFileEntry;

   delete fBrowseButton; 
   delete fPrintToFrame; 
   delete fPrinterFrame; 
   delete fBrowseFrame;
   delete fButtonFrame;
   delete fDestGFrame; 
}

/* 
 * Called when window is closed via the window manager.
 */
void PrintSetup::CloseWindow() 
{
   delete this;
}
/*
 * Create the dialog GUI
 */
void PrintSetup::CreateDisplay() 
{
  fDestGFrame = new TGGroupFrame(this, "Print Destination", kVerticalFrame);
  fDestGFrame->SetLayoutManager(new TGMatrixLayout(fDestGFrame,0,1,5,5));
  AddFrame(fDestGFrame, 
           new TGLayoutHints(kLHintsTop|kLHintsLeft|kLHintsExpandX,5, 5, 5, 5));

  fPrintToFrame = new TGCompositeFrame(fDestGFrame, 400, 20, kHorizontalFrame);
  fDestGFrame->AddFrame(fPrintToFrame, 
             new TGLayoutHints(kLHintsLeft|kLHintsTop,1,1,1,1));

  fPrintToFrame->AddFrame(new TGLabel(fPrintToFrame, new TGHotString("Print To:")),
           new TGLayoutHints(kLHintsLeft|kLHintsTop,5,5,0,0));
  fDestRB[0] = new TGRadioButton(fPrintToFrame, new TGHotString("Printer"), 21);
  fDestRB[1] = new TGRadioButton(fPrintToFrame, new TGHotString("File"),    22);
  for (int i = 0; i < 2; i++) {
    fPrintToFrame->AddFrame(fDestRB[i], 
             new TGLayoutHints(kLHintsLeft|kLHintsTop,50,5,0,0));
    fDestRB[i]->Associate(this);
  }

  fPrinterFrame = new TGCompositeFrame(fDestGFrame, 400, 30, kHorizontalFrame);
  fDestGFrame->AddFrame(fPrinterFrame, 
      new TGLayoutHints(kLHintsLeft|kLHintsTop,1,1,0,0));
  fPrinterFrame->AddFrame(new TGLabel(fPrinterFrame, new TGHotString("Printer Name:")),
                 new TGLayoutHints(kLHintsLeft|kLHintsTop,5,5,0,0));
  fPrinterEntry = new TGTextEntry(fPrinterFrame, fPrinterBuf = new TGTextBuffer(100));
  fPrinterEntry->Resize(250, fPrinterEntry->GetDefaultHeight());
  fPrinterFrame->AddFrame(fPrinterEntry,
             new TGLayoutHints(kLHintsLeft|kLHintsTop,5,1,0,0));

  fBrowseFrame = new TGCompositeFrame(fDestGFrame, 400, 30, kHorizontalFrame);
  fDestGFrame->AddFrame(fBrowseFrame,
               new TGLayoutHints(kLHintsLeft|kLHintsTop|kLHintsExpandX,1,1,1,1));
  fBrowseFrame->AddFrame(new TGLabel(fBrowseFrame, new TGHotString("File Name:")),
                 new TGLayoutHints(kLHintsLeft|kLHintsTop,5,5,1,1));
  fFileEntry = new TGTextEntry(fBrowseFrame, fFileBuf = new TGTextBuffer(100));
  fFileEntry->Resize(250, fFileEntry->GetDefaultHeight());
  fBrowseFrame->AddFrame(fFileEntry,
             new TGLayoutHints(kLHintsLeft|kLHintsTop,20,2,1,4));

  fBrowseButton = new TGTextButton(fBrowseFrame, "&Browse", M_PRINTSETUP_BROWSE);
  fBrowseButton->Associate(this);
  fBrowseButton->SetToolTipText("Browse files");
  // fBrowseButton->Resize(100, fBrowseButton->GetDefaultHeight());
  fBrowseFrame->AddFrame(fBrowseButton,
       new TGLayoutHints(kLHintsTop | kLHintsLeft,5, 2, 0, 5));
  setDefaultState();

  /* Buttons at bottom right */
  fButtonFrame = new TGHorizontalFrame(this, 120, 20, kFixedWidth);
  fL1 = new TGLayoutHints(kLHintsTop | kLHintsLeft | kLHintsExpandX,
                                        3, 3, 2, 2);
  fL2 = new TGLayoutHints(kLHintsBottom | kLHintsRight, 3, 3, 2, 2);

  fPrintButton = new TGTextButton(fButtonFrame, "&Print", M_PRINTSETUP_PRINT);
  fPrintButton->Associate(this);
  fPrintButton->SetToolTipText("Send file to the printer");
  fButtonFrame->AddFrame(fPrintButton, fL1);

  fCancelButton = new TGTextButton(fButtonFrame, "&Cancel", M_PRINTSETUP_CANCEL);
  fCancelButton->Associate(this);
  fCancelButton->SetToolTipText("Cancel printing dialog");
  fButtonFrame->AddFrame(fCancelButton, fL1);
  
  fButtonFrame->Resize(150, fCancelButton->GetDefaultHeight());
  AddFrame(fButtonFrame,fL2);
}
/* 
 *  Set Default state of the Buttons 
 */
void PrintSetup::setDefaultState()
{
  /* Printer Option is the default one */
  fDestRB[0]->SetState(kButtonDown); 
  fPrinterEntry->SetState(kTRUE);
  /* File Option should be disabled */
  fDestRB[1]->SetState(kButtonUp); 
  fFileEntry->SetState(kFALSE);
  fBrowseButton->SetState(kButtonDisabled); 
  *fFlagVal = 1; 
}
/* 
 *  Process Messages
 */
Bool_t PrintSetup::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2) 
{
   switch (GET_MSG(msg)) {

     case kC_COMMAND:
       switch (GET_SUBMSG(msg)) {

         case kCM_BUTTON:
           switch (parm1) {

             case M_PRINTSETUP_PRINT:
               {
                 *fRetCode = kTRUE;
                 readOutputName();
                 CloseWindow();
               }
               break;

             case M_PRINTSETUP_CANCEL:
               {
                 *fRetCode = kFALSE;
                 CloseWindow();
               }
               break;

             case M_PRINTSETUP_BROWSE:
               {
                  TGFileInfo fi;

		  fi.fFileTypes = printFileTypes;
                  new TGFileDialog(fClient->GetRoot(), this, kFDOpen, &fi);
                  if (!fi.fFilename) 
                    return kFALSE;
                  else {
                    char* file_name = fi.fFilename;
                    fFileEntry->SetText(file_name);
                  }
               }               
               break;
           }
         case kCM_RADIOBUTTON:

           switch (parm1) {

              case 21:
                setDefaultState();
                *fFlagVal = 1;
                break;

              case 22:
                fBrowseButton->SetState(kButtonUp);
                fFileEntry->SetState(kTRUE);
                fDestRB[0]->SetState(kButtonUp);
                fPrinterEntry->SetState(kFALSE);
                *fFlagVal = 2;
                break;

           }
       }
   }
   return kTRUE;
}
/*
 * Read Output Printer/File Name
 */
void PrintSetup::readOutputName() 
{
  if (fDestRB[0]->GetState() == kButtonDown)
    strcpy(fOutputName,(char *) fPrinterEntry->GetText());
  if (fDestRB[1]->GetState() == kButtonDown)
    strcpy(fOutputName,(char *) fFileEntry->GetText());
}
